# $Id: drilldownconfig.py 3919 2010-08-31 06:51:38Z place $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.9.1/src/drilldownconfig.py $
import pprint
from meresco.core import Observable
import traceback

class DrilldownConfig(Observable):
    def __init__(self, infra):
        Observable.__init__(self)
        self.infra = infra
        self.logger = infra.logging.getLogger('meresco.drilldownconfig')
        self.makeDrilldownDict()

    def getDrilldownFields(self,name):
        resultList = []
        configRoot = self.infra.configRoot
        serverConfig    = self.infra.configRoot
        section = "drilldownFields"
        fieldsList = configRoot.getValueAsList(name, section = section)

        for name in fieldsList:
            if name == '':
                continue
            if not name in self.fieldFeatures:
                s  = 'Configuration ERROR: "' + configRoot.getSectionFullname('drilldown')+'useDrilldownFields"\n'
                s += 'contains invalid field: "' + name + '"\n'
                s += 'This should match config with: "' + serverConfig.getSectionFullname('indices') + 'drilldownFieldnames"\n'
                self.logger.error(s)
                continue
                tupe = configRoot.get('drillDownFieldsDefaults', default = '10 5 True', section = 'drilldownFields')

            else:
                tupe = self.fieldFeatures[name]

            (nrRequested, nrDisplayed, boolean) = tupe
            resultList.append ( (name, nrRequested, boolean) )

        return resultList

    def getFieldFeatures(self):
        return self.fieldFeatures

    def getMultiLevelDrilldownFields(self):
        configRoot = self.infra.configRoot
        drildownConfig = configRoot
        drillDict = configRoot.getSectionDict('multilevelDrilldownFields')
        cleanDict = {}

        for level in drillDict:
            values = drillDict[level].split('\n')
            fields = []
            for field in values:
                if field == '':
                    continue

                if not field in self.fieldFeatures:
                    s  = 'Drilldownconfig ERROR: "' + drilldownConfig.getSectionFullname('multilevelDrilldownFields')
                    s += 'contains invalid field: "' + field + '"\n'
                    s += 'This should match config in: "' + serverConfig.getSectionFullname('untokenizedFields') + 'drilldownFieldnames"\n'
                    self.logger.error(s)
                    continue
                fields.append(field)

            cleanDict [level] = fields

#        self.logger.debug(self.infra.prpt("cleanDict",cleanDict))
        return cleanDict

    def makeDrilldownDict(self):
        configRoot = self.infra.configRoot
        drilldownConfig = self.infra.configRoot
        serverConfig    = self.infra.configRoot

        fieldFeatures = {}
        drilldownDefaults = drilldownConfig.get('drillDownFieldsDefaults', default = '10 5 True', section = 'drilldownFields')
        (defaultNrRequested, defaultNrDisplayed, defaultBoolean) = drilldownDefaults.split()

        allowedFieldsList = configRoot.getValueAsList('drilldownFieldnames', section = 'untokenizedFields')
        useFieldsList = configRoot.getValueAsList('useDrilldownFields', section = 'drilldownFields')

        list = []
        for line in useFieldsList:
            if line == "":
                continue
            nameList = line.split()

            name = nameList[0]
            name = name.strip()

            if not name in allowedFieldsList:
                s  = 'Drilldownconfig ERROR: "' + drilldownConfig.getSectionFullname('drilldownFields') + 'useDrilldownFields"\n'
                s += 'contains invalid field: "' + name + '"\n'
                s += 'This should match config in: "' + serverConfig.getSectionFullname('untokenizedFields') + 'drilldownFieldnames"\n'
                self.logger.error(s)
                continue

            boolean = defaultBoolean
            nrRequested = defaultNrRequested
            nrDisplayed = defaultNrDisplayed
            if len (nameList) == 1:
                nrRequested = defaultNrRequested
                nrDisplayed = defaultNrDisplayed
                boolean = defaultBoolean
            elif len (nameList) == 4:
                # ternary expressions in python :(
                nrRequested = nameList[1] if nameList[1] else defaultNrRequested
                nrDisplayed = nameList[2] if nameList[2] else defaultNrDisplayed
                boolean     = nameList[3] if nameList[3] else defaultBoolean

            if boolean == 'True':
                boolean = True
            else:
                boolean = False

            nrRequested = int (nrRequested)
            nrDisplayed = int(nrDisplayed)

            self.logger.debug('name: ' + name + ' --nrRequested: ' + str(nrRequested)  )
            tupe = (name, nrRequested, boolean)
            list.append(tupe)

            if (nrRequested and nrDisplayed > 0 and nrDisplayed > nrRequested):
                self.logger.warning ('Number of displayed items (' + str(nrDisplayed) + ') exceeds number of requested items ('+ str(nrRequested) +') for "' + name + '"')
                nrDisplayed = nrRequested

            fieldFeatures[ name ] = ( nrRequested, nrDisplayed, boolean)

        self.fieldFeatures = fieldFeatures
        return fieldFeatures
